@@ -290,7 +290,7 @@ create_sys_view (ProcData *procdata)
 					 G_CALLBACK(size_request), &cpu_size);
 
 		color_picker = gtk_color_button_new_with_color (
-			&load_graph_get_colors(cpu_graph)[2+i]);
+			&load_graph_get_colors(cpu_graph)[i]);
 
 		g_signal_connect (G_OBJECT (color_picker), "color_set",
 			    G_CALLBACK (cb_cpu_color_changed), GINT_TO_POINTER (i));
@@ -347,7 +347,7 @@ create_sys_view (ProcData *procdata)
 			    FALSE, FALSE, 0);
 
 	color_picker = gtk_color_button_new_with_color (
-		&load_graph_get_colors(mem_graph)[2]);
+		&load_graph_get_colors(mem_graph)[0]);
 	g_signal_connect (G_OBJECT (color_picker), "color_set",
 			    G_CALLBACK (cb_mem_color_changed), procdata);
 	gtk_table_attach (GTK_TABLE (table), color_picker, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
@@ -406,7 +406,7 @@ create_sys_view (ProcData *procdata)
 			  0);
 
 	color_picker = gtk_color_button_new_with_color (
-		&load_graph_get_colors(mem_graph)[3]);
+		&load_graph_get_colors(mem_graph)[1]);
 	g_signal_connect (G_OBJECT (color_picker), "color_set",
 			    G_CALLBACK (cb_swap_color_changed), procdata);
 	gtk_table_attach (GTK_TABLE (table), color_picker, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
@@ -496,7 +496,7 @@ create_sys_view (ProcData *procdata)
 			    FALSE, FALSE, 0);
 
 	color_picker = gtk_color_button_new_with_color (
-		&load_graph_get_colors(net_graph)[2]);
+		&load_graph_get_colors(net_graph)[0]);
 	g_signal_connect (G_OBJECT (color_picker), "color_set",
 			    G_CALLBACK (cb_net_in_color_changed), procdata);
 	gtk_table_attach (GTK_TABLE (table), color_picker, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
@@ -540,7 +540,7 @@ create_sys_view (ProcData *procdata)
 			  0);
 
 	color_picker = gtk_color_button_new_with_color (
-		&load_graph_get_colors(net_graph)[3]);
+		&load_graph_get_colors(net_graph)[1]);
 	g_signal_connect (G_OBJECT (color_picker), "color_set",
 			    G_CALLBACK (cb_net_out_color_changed), procdata);
 	gtk_table_attach (GTK_TABLE (table), color_picker, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
