@@ -43,6 +43,21 @@
 #include "util.h"
 #include "gconf-keys.h"
 
+
+
+ProcData::ProcData()
+  : tree(NULL),
+    cpu_graph(NULL),
+    mem_graph(NULL),
+    net_graph(NULL),
+    selected_process(NULL),
+    timeout(0),
+    disk_timeout(0),
+    cpu_total_time(1),
+    cpu_total_time_last(1)
+{ }
+
+
 ProcData* ProcData::get_instance()
 {
   static ProcData instance;
@@ -57,9 +72,14 @@ tree_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data
 	GConfValue *value = gconf_entry_get_value (entry);
 	
 	procdata->config.show_tree = gconf_value_get_bool (value);
+
+	g_object_set(G_OBJECT(procdata->tree),
+		     "show-expanders", procdata->config.show_tree,
+		     NULL);
+
 	proctable_clear_tree (procdata);
+
 	proctable_update_all (procdata);
-	
 }
 
 static void
@@ -234,14 +254,6 @@ procman_data_new (GConfClient *client)
 	glibtop_cpu cpu;
 
 	pd = ProcData::get_instance();
-	
-	pd->tree = NULL;
-	pd->selected_process = NULL;
-	pd->timeout = 0;
-	pd->cpu_graph = NULL;
-	pd->mem_graph = NULL;
-	pd->net_graph = NULL;
-	pd->disk_timeout = 0;
 
 	/* username is usually 8 chars long
 	   for caching, we create chunks of 128 chars */
