@@ -52,16 +52,9 @@
 #include "util.h"
 #include "interface.h"
 #include "selinux.h"
-extern "C" {
-#include "e_date.h"
-}
-
-static guint64 total_time = 1;
-static guint64 total_time_last = 1;
-
-
 
 ProcInfo::List ProcInfo::all;
+std::map<pid_t, guint64> ProcInfo::cpu_times;
 
 
 ProcInfo* ProcInfo::find(pid_t pid)
@@ -73,61 +66,6 @@ ProcInfo* ProcInfo::find(pid_t pid)
 
 
 
-static gint
-sort_ints (GtkTreeModel *model, GtkTreeIter *itera, GtkTreeIter *iterb, gpointer data)
-{
-	ProcInfo *infoa = NULL, *infob = NULL;
-	const gint col = GPOINTER_TO_INT (data);
-
-	gtk_tree_model_get (model, itera, COL_POINTER, &infoa, -1);
-	gtk_tree_model_get (model, iterb, COL_POINTER, &infob, -1);
-
-	g_assert(infoa);
-	g_assert(infob);
-
-	switch (col) {
-	case COL_CPU_TIME:
-		return PROCMAN_RCMP(infoa->cpu_time_last, infob->cpu_time_last);
-
-	case COL_START_TIME:
-		return PROCMAN_CMP(infoa->start_time, infob->start_time);
-
-	case COL_CPU:
-		return PROCMAN_RCMP(infoa->pcpu, infob->pcpu);
-	default:
-		g_assert_not_reached();
-		return 0;
-	}
-}
-
-
-
-static GtkWidget *
-create_proctree(GtkTreeModel *model)
-{
-	GtkWidget *proctree;
-	GtkWidget* (*sexy_new)(void);
-	void (*sexy_set_column)(void*, guint);
-
-
-	if (FALSE && load_symbols("libsexy.so",
-			 "sexy_tree_view_new", &sexy_new,
-			 "sexy_tree_view_set_tooltip_label_column", &sexy_set_column,
-			 NULL)) {
-		proctree = sexy_new();
-		gtk_tree_view_set_model(GTK_TREE_VIEW(proctree), model);
-		sexy_set_column(proctree, COL_TOOLTIP);
-	} else {
-		proctree = gtk_tree_view_new_with_model(model);
-	}
-
-	return proctree;
-}
-
-
-
-
-
 static void
 set_proctree_reorderable(ProcData *procdata)
 {
@@ -300,15 +238,15 @@ proctable_new (ProcData * const procdata)
 	model = gtk_tree_store_new (NUM_COLUMNS,
 				    G_TYPE_STRING,	/* Process Name */
 				    G_TYPE_STRING,	/* User		*/
-				    G_TYPE_STRING,	/* Status	*/
+				    G_TYPE_UINT,	/* Status	*/
 				    G_TYPE_ULONG,	/* VM Size	*/
 				    G_TYPE_ULONG,	/* Resident Memory */
 				    G_TYPE_ULONG,	/* Writable Memory */
 				    G_TYPE_ULONG,	/* Shared Memory */
 				    G_TYPE_ULONG,	/* X Server Memory */
 				    G_TYPE_UINT,	/* % CPU	*/
-				    G_TYPE_STRING,	/* CPU time	*/
-				    G_TYPE_STRING,	/* Started	*/
+				    G_TYPE_UINT64,	/* CPU time	*/
+				    G_TYPE_ULONG,	/* Started	*/
 				    G_TYPE_INT,		/* Nice		*/
 				    G_TYPE_UINT,	/* ID		*/
 				    G_TYPE_STRING,	/* Security Context */
@@ -319,7 +257,11 @@ proctable_new (ProcData * const procdata)
 				    G_TYPE_STRING	/* Sexy tooltip */
 		);
 
-	proctree = create_proctree(GTK_TREE_MODEL(model));
+	proctree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
+	gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (proctree), COL_TOOLTIP);
+	g_object_set(G_OBJECT(proctree),
+		     "show-expanders", procdata->config.show_tree,
+		     NULL);
 	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (proctree),
 					     search_equal_func,
 					     NULL,
@@ -368,18 +310,45 @@ proctable_new (ProcData * const procdata)
 
 		// type
 		switch (i) {
-		case COL_VMSIZE:
-		case COL_MEMRES:
-		case COL_MEMWRITABLE:
-		case COL_MEMSHARED:
 		case COL_MEMXSERVER:
-		case COL_MEM:
 		  gtk_tree_view_column_set_cell_data_func(col, cell,
 							  &procman::size_cell_data_func,
 							  GUINT_TO_POINTER(i),
 							  NULL);
 		  break;
 
+		case COL_VMSIZE:
+		case COL_MEMRES:
+		case COL_MEMSHARED:
+		case COL_MEM:
+		case COL_MEMWRITABLE:
+		  gtk_tree_view_column_set_cell_data_func(col, cell,
+							  &procman::size_na_cell_data_func,
+							  GUINT_TO_POINTER(i),
+							  NULL);
+		  break;
+
+		case COL_CPU_TIME:
+		  gtk_tree_view_column_set_cell_data_func(col, cell,
+							  &procman::duration_cell_data_func,
+							  GUINT_TO_POINTER(i),
+							  NULL);
+		  break;
+
+		case COL_START_TIME:
+		  gtk_tree_view_column_set_cell_data_func(col, cell,
+							  &procman::time_cell_data_func,
+							  GUINT_TO_POINTER(i),
+							  NULL);
+		  break;
+
+		case COL_STATUS:
+		  gtk_tree_view_column_set_cell_data_func(col, cell,
+							  &procman::status_cell_data_func,
+							  GUINT_TO_POINTER(i),
+							  NULL);
+		  break;
+
 		default:
 		  gtk_tree_view_column_set_attributes(col, cell, "text", i, NULL);
 		  break;
@@ -416,23 +385,6 @@ proctable_new (ProcData * const procdata)
 
 	gtk_container_add (GTK_CONTAINER (scrolled), proctree);
 
-
-	for(i = COL_NAME; i <= COL_MEM; i++)
-	{
-		switch(i)
-		{
-		case COL_CPU_TIME:
-		case COL_START_TIME:
-		case COL_CPU:
-			gtk_tree_sortable_set_sort_func (
-				GTK_TREE_SORTABLE (model),
-				i,
-				sort_ints,
-				GINT_TO_POINTER(i),
-				NULL);
-		}
-	}
-
 	procdata->tree = proctree;
 
 	set_proctree_reorderable(procdata);
@@ -470,35 +422,6 @@ ProcInfo::~ProcInfo()
 }
 
 
-static void
-get_process_status (ProcInfo *info, const glibtop_proc_state *buf)
-{
-
-	switch(buf->state)
-	{
-	case GLIBTOP_PROCESS_RUNNING:
-		info->status = _("Running");
-		break;
-
-	case GLIBTOP_PROCESS_STOPPED:
-		info->status = _("Stopped");
-		break;
-
-	case GLIBTOP_PROCESS_ZOMBIE:
-		info->status = _("Zombie");
-		break;
-
-	case GLIBTOP_PROCESS_UNINTERRUPTIBLE:
-		info->status = _("Uninterruptible");
-		break;
-
-	default:
-		info->status = _("Sleeping");
-		break;
-	}
-}
-
-
 static void
 get_process_name (ProcInfo *info,
 		  const gchar *cmd, const gchar *args)
@@ -591,50 +514,12 @@ get_process_memory_info(ProcInfo *info)
 
 	get_process_memory_writable(info);
 
-	info->mem = info->memxserver + info->memwritable;
+	// fake the smart memory column if writable is not available
+	info->mem = info->memxserver + (info->memwritable ? info->memwritable : info->memres);
 }
 
 
 
-static inline unsigned divide(unsigned *q, unsigned *r, unsigned d)
-{
-	*q = *r / d;
-	*r = *r % d;
-	return *q != 0;
-}
-
-/*
- * @param d: duration in centiseconds
- * @type d: unsigned
- */
-static char *
-format_duration_for_display(unsigned centiseconds)
-{
-	unsigned weeks = 0, days = 0, hours = 0, minutes = 0, seconds = 0;
-
-	(void)(divide(&seconds, &centiseconds, 100)
-	       && divide(&minutes, &seconds, 60)
-	       && divide(&hours, &minutes, 60)
-	       && divide(&days, &hours, 24)
-	       && divide(&weeks, &days, 7));
-
-	if (weeks)
-		/* xgettext: weeks, days */
-		return g_strdup_printf(_("%uw%ud"), weeks, days);
-
-	if (days)
-		/* xgettext: days, hours (0 -> 23) */
-		return g_strdup_printf(_("%ud%02uh"), days, hours);
-
-	if (hours)
-		/* xgettext: hours (0 -> 23), minutes, seconds */
-		return g_strdup_printf(_("%u:%02u:%02u"), hours, minutes, seconds);
-
-	/* xgettext: minutes, seconds, centiseconds */
-	return g_strdup_printf(_("%u:%02u.%02u"), minutes, seconds, centiseconds);
-}
-
-
 static void
 update_info_mutable_cols(ProcInfo *info)
 {
@@ -642,34 +527,20 @@ update_info_mutable_cols(ProcInfo *info)
 	GtkTreeModel *model;
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(procdata->tree));
 
-	gchar *cpu_time, *start_time;
+	using procman::tree_store_update;
 
-	// expects centiseconds
-	cpu_time = format_duration_for_display(100 * info->cpu_time_last / procdata->frequency);
-
-	/* FIXME: does it worths it to display relative to $now date ?
-	   absolute date wouldn't required to be updated on every refresh */
-	start_time = procman_format_date_for_display(info->start_time);
-
-	gtk_tree_store_set(GTK_TREE_STORE(model), &info->node,
-			    COL_STATUS, info->status,
-			    COL_USER, info->user,
-			    COL_VMSIZE, info->vmsize,
-			    COL_MEMRES, info->memres,
-			    COL_MEMWRITABLE, info->memwritable,
-			    COL_MEMSHARED, info->memshared,
-			    COL_MEMXSERVER, info->memxserver,
-			    COL_CPU, guint(info->pcpu),
-			    COL_CPU_TIME, cpu_time,
-			    COL_START_TIME, start_time,
-			    COL_NICE, gint(info->nice),
-			    COL_MEM, info->mem,
-			    -1);
-
-	/* FIXME: We don't bother updating COL_SECURITYCONTEXT as it can never change.
-	   even on fork ? */
-	g_free (cpu_time);
-	g_free(start_time);
+	tree_store_update(model, &info->node, COL_STATUS, info->status);
+	tree_store_update(model, &info->node, COL_USER, info->user);
+	tree_store_update(model, &info->node, COL_VMSIZE, info->vmsize);
+	tree_store_update(model, &info->node, COL_MEMRES, info->memres);
+	tree_store_update(model, &info->node, COL_MEMWRITABLE, info->memwritable);
+	tree_store_update(model, &info->node, COL_MEMSHARED, info->memshared);
+	tree_store_update(model, &info->node, COL_MEMXSERVER, info->memxserver);
+	tree_store_update(model, &info->node, COL_CPU, info->pcpu);
+	tree_store_update(model, &info->node, COL_CPU_TIME, info->cpu_time);
+	tree_store_update(model, &info->node, COL_START_TIME, info->start_time);
+	tree_store_update(model, &info->node, COL_NICE, info->nice);
+	tree_store_update(model, &info->node, COL_MEM, info->mem);
 }
 
 
@@ -701,14 +572,8 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata, bool forced = false)
 	else
 		gtk_tree_store_insert (GTK_TREE_STORE (model), &info->node, NULL, 0);
 
-	/* COL_POINTER must be set first, because GtkTreeStore
-	 * will call sort_ints as soon as we set the column
-	 * that we're sorting on.
-	 */
-
 	gtk_tree_store_set (GTK_TREE_STORE (model), &info->node,
 			    COL_POINTER, info,
-			    COL_PIXBUF, info->pixbuf->gobj(),
 			    COL_NAME, info->name,
 			    COL_ARGS, info->arguments,
 			    COL_TOOLTIP, info->tooltip,
@@ -716,6 +581,8 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata, bool forced = false)
 			    COL_SECURITYCONTEXT, info->security_context,
 			    -1);
 
+	procdata->pretty_table.set_icon(*info);
+
 	procman_debug("inserted %d%s", info->pid, (forced ? " (forced)" : ""));
 }
 
@@ -768,22 +635,22 @@ update_info (ProcData *procdata, ProcInfo *info)
 	glibtop_proc_time proctime;
 
 	glibtop_get_proc_state (&procstate, info->pid);
+	info->status = procstate.state;
+
 	glibtop_get_proc_uid (&procuid, info->pid);
 	glibtop_get_proc_time (&proctime, info->pid);
 
-	get_process_status (info, &procstate);
-
 	get_process_memory_info(info);
 
 	get_process_user(procdata, info, procstate.uid);
 
-	info->pcpu = (proctime.rtime - info->cpu_time_last) * 100 / total_time;
+	info->pcpu = (proctime.rtime - info->cpu_time) * 100 / procdata->cpu_total_time;
 	info->pcpu = MIN(info->pcpu, 100);
 
 	if (procdata->config.solaris_mode)
 	  info->pcpu /= procdata->config.num_cpus;
 
-	info->cpu_time_last = proctime.rtime;
+	ProcInfo::cpu_times[info->pid] = info->cpu_time = proctime.rtime;
 	info->nice = procuid.nice;
 	info->ppid = procuid.ppid;
 }
@@ -792,9 +659,7 @@ update_info (ProcData *procdata, ProcInfo *info)
 ProcInfo::ProcInfo(pid_t pid)
   : tooltip(NULL),
     name(NULL),
-    user(NULL),
     arguments(NULL),
-    status(NULL),
     security_context(NULL),
     pid(pid),
     uid(-1)
@@ -812,16 +677,24 @@ ProcInfo::ProcInfo(pid_t pid)
 	/* FIXME : wrong. name and arguments may change with exec* */
 	get_process_name (info, procstate.cmd, arguments[0]);
 
-	info->tooltip = g_strjoinv(" ", arguments);
-	info->arguments = g_strescape(info->tooltip, "\\\"");
+	char* tooltip = g_strjoinv(" ", arguments);
+	info->tooltip = g_markup_escape_text(tooltip, -1);
+
+	info->arguments = g_strescape(tooltip, "\\\"");
+	g_free(tooltip);
 	g_strfreev(arguments);
 
-	info->cpu_time_last = proctime.rtime;
+	guint64 cpu_time = proctime.rtime;
+	std::map<pid_t, guint64>::iterator it(ProcInfo::cpu_times.find(pid));
+	if (it != ProcInfo::cpu_times.end())
+	  {
+	    if (proctime.rtime >= it->second)
+	      cpu_time = it->second;
+	  }
+	info->cpu_time = cpu_time;
 	info->start_time = proctime.start_time;
 
 	get_process_selinux_context (info);
-
-	info->pixbuf = ProcData::get_instance()->pretty_table.get_icon(info->name, pid);
 }
 
 
@@ -988,8 +861,8 @@ proctable_update_list (ProcData * const procdata)
 	/* FIXME: total cpu time elapsed should be calculated on an individual basis here
 	** should probably have a total_time_last gint in the ProcInfo structure */
 	glibtop_get_cpu (&cpu);
-	total_time = MAX(cpu.total - total_time_last, 1);
-	total_time_last = cpu.total;
+	procdata->cpu_total_time = MAX(cpu.total - procdata->cpu_total_time_last, 1);
+	procdata->cpu_total_time_last = cpu.total;
 
 	refresh_list (procdata, pid_list, proclist.number);
 
